<?php

/**
 * @wordpress-plugin
 * Plugin Name:       AI Image Generator
 * Plugin URI:        https://upcasted.com/ai-image-generator/
 * Description:       A plugin that uses OpenAI Image Generation API to generate similar images based on an image or a text.
 * Version:           1.0.1
 * Author:            Upcasted
 * Author URI:        https://upcasted.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       ai-wp-upcasted
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! function_exists( 'aiwpupcasted_fs' ) ) {
    // Create a helper function for easy SDK access.
    function aiwpupcasted_fs() {
        global $aiwpupcasted_fs;

        if ( ! isset( $aiwpupcasted_fs ) ) {
            // Include Freemius SDK.
            require_once dirname(__FILE__) . '/freemius/start.php';

            $aiwpupcasted_fs = fs_dynamic_init( array(
                'id'                  => '12017',
                'slug'                => 'ai-image-generator',
                'type'                => 'plugin',
                'public_key'          => 'pk_b377f9d6f526b0e991c55aafc2f26',
                'is_premium'          => false,
                'has_addons'          => false,
                'has_paid_plans'      => false,
                'menu'                => array(
                    'slug'           => 'ai-wp-upcasted',
                    'first-path'     => 'upload.php?page=ai-wp-upcasted',
                    'account'        => false,
                    'contact'        => false,
                    'support'        => false,
                    'parent'         => array(
                        'slug' => 'upload.php',
                    ),
                ),
            ) );
        }

        return $aiwpupcasted_fs;
    }

    // Init Freemius.
    aiwpupcasted_fs();
    // Signal that SDK was initiated.
    do_action( 'aiwpupcasted_fs_loaded' );
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'AI_FOR_WORDPRESS_BY_UPCASTED_VERSION', '1.0.1' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-ai-wp-upcasted-activator.php
 */
function activate_ai_for_wordpress_by_upcasted() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-ai-wp-upcasted-activator.php';
	Ai_For_Wordpress_By_Upcasted_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-ai-wp-upcasted-deactivator.php
 */
function deactivate_ai_for_wordpress_by_upcasted() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-ai-wp-upcasted-deactivator.php';
	Ai_For_Wordpress_By_Upcasted_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_ai_for_wordpress_by_upcasted' );
register_deactivation_hook( __FILE__, 'deactivate_ai_for_wordpress_by_upcasted' );

require_once(plugin_dir_path(__FILE__) . '/lib/autoload.php');
//use Orhanerday\OpenAi\OpenAi;

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-ai-wp-upcasted.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_ai_for_wordpress_by_upcasted() {

	$plugin = new Ai_For_Wordpress_By_Upcasted();
	$plugin->run();

}
run_ai_for_wordpress_by_upcasted();
